@@ -1,7 +1,7 @@
 /*
-    Qalculate    
+    Qalculate (GTK+ UI)
 
-    Copyright (C) 2003-2007  Niklas Knutsson (nq@altern.org)
+    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna_k@fmgirl.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,10 +16,6 @@
 
 #include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <glade/glade.h>
-#ifdef HAVE_LIBGNOME
-#include <libgnome/libgnome.h>
-#endif
 #include <unistd.h>
 
 #include "support.h"
@@ -27,7 +23,7 @@
 #include "callbacks.h"
 #include "main.h"
 
-MathStructure *mstruct, *matrix_mstruct, *parsed_mstruct, *parsed_tostruct;
+MathStructure *mstruct, *matrix_mstruct, *parsed_mstruct, *parsed_tostruct, *displayed_mstruct;
 bool prev_result_approx;
 string *parsed_to_str;
 KnownVariable *vans[5];
@@ -42,21 +38,20 @@ GtkWidget *units_window;
 string selected_unit_category;
 string selected_unit_selector_category;
 Unit *selected_unit, *selected_to_unit;
-bool load_global_defs, fetch_exchange_rates_at_startup, first_time, first_qalculate_run;
-GtkWidget *omToUnit_menu;
-GdkPixmap *pixmap_result;
+bool load_global_defs, fetch_exchange_rates_at_startup, first_time, showing_first_time_message;
+cairo_surface_t *surface_result;
 GdkPixbuf *pixbuf_result;
-extern bool b_busy;
+extern bool b_busy, b_busy_command, b_busy_result, b_busy_expression;
 extern vector<string> recent_functions_pre;
 extern vector<string> recent_variables_pre;
 extern vector<string> recent_units_pre;
 extern GtkWidget *expression;
 extern PrintOptions printops;
 
-GladeXML *main_glade, *about_glade, *argumentrules_glade, *csvimport_glade, *csvexport_glade, *setbase_glade, *datasetedit_glade, *datasets_glade, *decimals_glade;
-GladeXML *functionedit_glade, *functions_glade, *matrixedit_glade, *matrix_glade, *namesedit_glade, *nbases_glade, *plot_glade, *precision_glade;
-GladeXML *preferences_glade, *unit_glade, *unitedit_glade, *units_glade, *unknownedit_glade, *variableedit_glade, *variables_glade;
-GladeXML *periodictable_glade;
+GtkBuilder *main_builder, *argumentrules_builder, *csvimport_builder, *csvexport_builder, *setbase_builder, *datasetedit_builder, *datasets_builder, *decimals_builder;
+GtkBuilder *functionedit_builder, *functions_builder, *matrixedit_builder, *matrix_builder, *namesedit_builder, *nbases_builder, *plot_builder, *precision_builder;
+GtkBuilder *preferences_builder, *unit_builder, *unitedit_builder, *units_builder, *unknownedit_builder, *variableedit_builder, *variables_builder;
+GtkBuilder *periodictable_builder;
 
 FILE *view_pipe_r, *view_pipe_w, *command_pipe_r, *command_pipe_w;
 pthread_t view_thread, command_thread;
@@ -66,13 +61,12 @@ bool command_thread_started;
 bool do_timeout, check_expression_position;
 gint expression_position;
 
-#ifdef HAVE_LIBGNOME
-static poptContext pctx;
-static struct poptOption options[] = {
-	{NULL, '\0', 0, NULL, 0, NULL, NULL}
+static const char **args;
+static GOptionEntry options[] = {
+	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY,
+	&args, NULL},
+	{NULL}
 };
-#endif
-
 int main (int argc, char **argv) {
 
 #ifdef ENABLE_NLS
@@ -81,36 +75,15 @@ int main (int argc, char **argv) {
 	textdomain (GETTEXT_PACKAGE);
 #endif
 
-#ifdef HAVE_LIBGNOME
+	setlocale(LC_ALL, "");
 
-	GnomeProgram *program = gnome_program_init("qalculate-gtk", VERSION, LIBGNOME_MODULE, argc, argv, GNOME_PARAM_POPT_TABLE, options, GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR, NULL);
+	GError *error = NULL;
 
-	char *icon = gnome_program_locate_file(program, GNOME_FILE_DOMAIN_APP_PIXMAP, "qalculate.png", TRUE, NULL);
+	gtk_init_with_args(&argc, &argv, NULL, options, GETTEXT_PACKAGE, &error);
 
-	if(icon) {
-		gtk_window_set_default_icon_from_file(icon, NULL);
-		g_free(icon);
-	}
-	
-	gtk_init(&argc, &argv);
-
-	g_object_get(G_OBJECT(program), GNOME_PARAM_POPT_CONTEXT, &pctx, NULL);
-
-
-#else
-
-
-	gtk_init(&argc, &argv);
-
-	gtk_window_set_default_icon_from_file(PACKAGE_DATA_DIR "/pixmaps/qalculate.png", NULL);
-
-#endif
-
-	glade_init();
+	gtk_window_set_default_icon_from_file(PACKAGE_DATA_DIR "/pixmaps/qalculate.png", &error);
 
 	string calc_arg;
-#ifdef HAVE_LIBGNOME
-	const char **args = poptGetArgs (pctx);
 	for(int i = 0; args && args[i]; i++) {
 		if(i > 1) {
 			calc_arg += " ";
@@ -122,36 +95,28 @@ int main (int argc, char **argv) {
 			calc_arg += args[i];
 		}
 	}
-	poptFreeContext (pctx);
-#else	
-	for(int i = 1; i < argc; i++) {
-		if(i > 1) {
-			calc_arg += " ";
-		}
-		if(strlen(argv[i]) >= 2 && ((argv[i][0] == '\"' && argv[i][strlen(argv[i]) - 1] == '\"') || (argv[i][0] == '\'' && argv[i][strlen(argv[i]) - 1] == '\''))) {
-			calc_arg += argv[i] + 1;
-			calc_arg.erase(calc_arg.length() - 1);
-		} else {
-			calc_arg += argv[i];
-		}
-	}
-#endif
-	b_busy = false;
+	b_busy = false; 
+	b_busy_result = false;
+	b_busy_expression = false;
+	b_busy_command = false;
 
-	main_glade = NULL; about_glade = NULL; argumentrules_glade = NULL; 
-	csvimport_glade = NULL; datasetedit_glade = NULL; datasets_glade = NULL; decimals_glade = NULL; functionedit_glade = NULL; 
-	functions_glade = NULL; matrixedit_glade = NULL; matrix_glade = NULL; namesedit_glade = NULL; nbases_glade = NULL; plot_glade = NULL; 
-	precision_glade = NULL; preferences_glade = NULL; unit_glade = NULL; 
-	unitedit_glade = NULL; units_glade = NULL; unknownedit_glade = NULL; variableedit_glade = NULL; 
-	variables_glade = NULL;	csvexport_glade = NULL; setbase_glade = NULL; periodictable_glade = NULL;
+	main_builder = NULL; argumentrules_builder = NULL; 
+	csvimport_builder = NULL; datasetedit_builder = NULL; datasets_builder = NULL; decimals_builder = NULL; functionedit_builder = NULL; 
+	functions_builder = NULL; matrixedit_builder = NULL; matrix_builder = NULL; namesedit_builder = NULL; nbases_builder = NULL; plot_builder = NULL; 
+	precision_builder = NULL; preferences_builder = NULL; unit_builder = NULL; 
+	unitedit_builder = NULL; units_builder = NULL; unknownedit_builder = NULL; variableedit_builder = NULL; 
+	variables_builder = NULL;	csvexport_builder = NULL; setbase_builder = NULL; periodictable_builder = NULL;
 
 	//create the almighty Calculator object
 	new Calculator();
-
+	
+	CALCULATOR->setExchangeRatesWarningEnabled(false);
+	
 	//load application specific preferences
 	load_preferences();
 
 	mstruct = new MathStructure();
+	displayed_mstruct = new MathStructure();
 	parsed_mstruct = new MathStructure();
 	parsed_tostruct = new MathStructure();
 	parsed_tostruct->setUndefined();
@@ -166,45 +131,54 @@ int main (int argc, char **argv) {
 	create_main_window();
 
 	while(gtk_events_pending()) gtk_main_iteration();
+	
+	showing_first_time_message = first_time;
 
 	if(!calc_arg.empty()) {
-		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(main_glade, "expression")), calc_arg.c_str());
-	} else if(first_time) {
-		PangoLayout *layout = gtk_widget_create_pango_layout(glade_xml_get_widget(main_glade, "resultview"), NULL);
-		gint w = 0, h = 0;
-		pango_layout_set_markup(layout, _("Enter a mathematical expression above. Ex. 5 + 2 / 3"), -1);
-		pango_layout_get_pixel_size(layout, &w, &h);
-		gdk_draw_layout(GDK_DRAWABLE(glade_xml_get_widget(main_glade, "resultview")->window), glade_xml_get_widget(main_glade, "resultview")->style->fg_gc[GTK_WIDGET_STATE(glade_xml_get_widget(main_glade, "expression"))], 0, 0, layout);	
+		gtk_entry_set_text(GTK_ENTRY(gtk_builder_get_object(main_builder, "expression")), calc_arg.c_str());
+	} else if(first_time) {		
+		PangoLayout *layout = gtk_widget_create_pango_layout(GTK_WIDGET(gtk_builder_get_object(main_builder, "resultview")), NULL);
+		GdkRGBA rgba;
+		cairo_t *cr = gdk_cairo_create(gtk_widget_get_window(GTK_WIDGET(gtk_builder_get_object(main_builder, "resultview"))));
+		pango_layout_set_markup(layout, _("<span font=\"10\">Type a mathematical expression above, e.g. \"5 + 2 / 3\",\nand press the enter key.</span>"), -1);
+		gtk_style_context_get_color(gtk_widget_get_style_context(GTK_WIDGET(gtk_builder_get_object(main_builder, "resultview"))), gtk_widget_get_state_flags(GTK_WIDGET(gtk_builder_get_object(main_builder, "resultview"))), &rgba);
+		cairo_move_to(cr, 6, 0);
+		gdk_cairo_set_source_rgba(cr, &rgba);
+		pango_cairo_show_layout(cr, layout);
 		g_object_unref(layout);
+		cairo_destroy(cr);
 	}
 
 	while(gtk_events_pending()) gtk_main_iteration();
 
 	//exchange rates
-	if(first_qalculate_run && canfetch) {
-		GtkWidget *edialog = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget (main_glade, "main_window")), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_WARNING, GTK_BUTTONS_YES_NO, _("You need to download exchange rates to be able to convert between different currencies. You can later get current exchange rates by selecting \"Update Exchange Rates\" under the File menu.\n\nDo you want to fetch exchange rates now from the Internet?"));
+	
+	if(fetch_exchange_rates_at_startup && canfetch) {
+		fetch_exchange_rates(5);
+		while(gtk_events_pending()) gtk_main_iteration();
+		CALCULATOR->loadExchangeRates();
+	} else if(!CALCULATOR->loadExchangeRates() && first_time && canfetch) {
+		GtkWidget *edialog = gtk_message_dialog_new(GTK_WINDOW(gtk_builder_get_object (main_builder, "main_window")), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_WARNING, GTK_BUTTONS_YES_NO, _("You need to download exchange rates to be able to convert between different currencies. You can later get current exchange rates by selecting “Update Exchange Rates” under the File menu.\n\nDo you want to fetch exchange rates now from the Internet?"));
 		int question_answer = gtk_dialog_run(GTK_DIALOG(edialog));
 		gtk_widget_destroy(edialog);
 		while(gtk_events_pending()) gtk_main_iteration();
 		if(question_answer == GTK_RESPONSE_YES) {
 			fetch_exchange_rates(5);
 			while(gtk_events_pending()) gtk_main_iteration();
+			CALCULATOR->loadExchangeRates();
 		}
-	} else if(fetch_exchange_rates_at_startup && canfetch) {
-		fetch_exchange_rates(5);
-		while(gtk_events_pending()) gtk_main_iteration();
 	}
 
-	CALCULATOR->loadExchangeRates();
+	
 
 	string ans_str = _("ans");
-	vans[0] = (KnownVariable*) CALCULATOR->addVariable(new KnownVariable(_("Temporary"), ans_str, m_undefined, _("Last Answer"), false));
+	vans[0] = (KnownVariable*) CALCULATOR->addVariable(new KnownVariable(CALCULATOR->temporaryCategory(), ans_str, m_undefined, _("Last Answer"), false));
 	vans[0]->addName(_("answer"));
 	vans[0]->addName(ans_str + "1");
-	vans[1] = (KnownVariable*) CALCULATOR->addVariable(new KnownVariable(_("Temporary"), ans_str + "2", m_undefined, _("Answer 2"), false));
-	vans[2] = (KnownVariable*) CALCULATOR->addVariable(new KnownVariable(_("Temporary"), ans_str + "3", m_undefined, _("Answer 3"), false));
-	vans[3] = (KnownVariable*) CALCULATOR->addVariable(new KnownVariable(_("Temporary"), ans_str + "4", m_undefined, _("Answer 4"), false));
-	vans[4] = (KnownVariable*) CALCULATOR->addVariable(new KnownVariable(_("Temporary"), ans_str + "5", m_undefined, _("Answer 5"), false));
+	vans[1] = (KnownVariable*) CALCULATOR->addVariable(new KnownVariable(CALCULATOR->temporaryCategory(), ans_str + "2", m_undefined, _("Answer 2"), false));
+	vans[2] = (KnownVariable*) CALCULATOR->addVariable(new KnownVariable(CALCULATOR->temporaryCategory(), ans_str + "3", m_undefined, _("Answer 3"), false));
+	vans[3] = (KnownVariable*) CALCULATOR->addVariable(new KnownVariable(CALCULATOR->temporaryCategory(), ans_str + "4", m_undefined, _("Answer 4"), false));
+	vans[4] = (KnownVariable*) CALCULATOR->addVariable(new KnownVariable(CALCULATOR->temporaryCategory(), ans_str + "5", m_undefined, _("Answer 5"), false));
 
 	//load global definitions
 	if(load_global_defs && !CALCULATOR->loadGlobalDefinitions()) {
@@ -225,10 +199,9 @@ int main (int argc, char **argv) {
 	selected_unit_category = _("All");
 	selected_unit = NULL;
 	selected_to_unit = NULL;
-	omToUnit_menu = NULL;
 	result_text = "0";
 	parsed_text = "0";
-	pixmap_result = NULL;
+	surface_result = NULL;
 	pixbuf_result = NULL;
 
 	//check for calculation errros regularly
@@ -239,8 +212,8 @@ int main (int argc, char **argv) {
 	expression_position = 1;
 	g_timeout_add(50, on_check_expression_position_timeout, NULL);
 	
-	gtk_widget_set_sensitive(glade_xml_get_widget (main_glade, "menu_item_plot_functions"), canplot);
-	gtk_widget_set_sensitive(glade_xml_get_widget (main_glade, "menu_item_fetch_exchange_rates"), canfetch);
+	gtk_widget_set_sensitive(GTK_WIDGET(gtk_builder_get_object (main_builder, "menu_item_plot_functions")), canplot);
+	gtk_widget_set_sensitive(GTK_WIDGET(gtk_builder_get_object (main_builder, "menu_item_fetch_exchange_rates")), canfetch);
 
 	//create dynamic menus
 	generate_units_tree_struct();
@@ -251,7 +224,9 @@ int main (int argc, char **argv) {
 	create_umenu();
 	//create_pmenu();	
 	create_umenu2();
-	create_pmenu2();			
+	create_pmenu2();
+	
+	update_unit_selector_tree();
 
 	for(int i = ((int) recent_functions_pre.size()) - 1; i >= 0; i--) {
 		function_inserted(CALCULATOR->getActiveFunction(recent_functions_pre[i]));
